package com.winhc.repal.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.winhc.common.base.ApiDataResult;
import com.winhc.common.enums.CodeMsg;
import com.winhc.common.exception.CommonException;
import com.winhc.common.model.base.VOPage;
import com.winhc.repal.cloud.DataMiddlewareService;
import com.winhc.repal.cloud.InCaseService;
import com.winhc.repal.cloud.dto.GetCompanyRiskNumDTO;
import com.winhc.repal.cloud.dto.GetFinanceDynamicNumDTO;
import com.winhc.repal.cloud.vo.CompanyRiskNumVO;
import com.winhc.repal.cloud.vo.DiagnosisRecordVO;
import com.winhc.repal.cloud.vo.FinanceDynamicNumVO;
import com.winhc.repal.entity.RepalBill;
import com.winhc.repal.entity.RepalBillDiagnosis;
import com.winhc.repal.entity.RepalChangeRecord;
import com.winhc.repal.entity.RepalCustomer;
import com.winhc.repal.enums.DeletedStatusEnum;
import com.winhc.repal.enums.RepalBillStatusEnum;
import com.winhc.repal.enums.RepalBillTypeEnum;
import com.winhc.repal.model.dto.GetRepalBillPageDTO;
import com.winhc.repal.model.vo.RepalBillDetailVO;
import com.winhc.repal.model.vo.RepalBillPageVO;
import com.winhc.repal.repository.RepalBillMapper;
import com.winhc.repal.service.*;
import com.winhc.repal.util.DiagnosisUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;


/**
 * @description RepalBill 接口实现类
 * @author Generator
 * @date 2022-04-08
 */
@Service
public class RepalBillServiceImpl extends ServiceImpl<RepalBillMapper, RepalBill> implements RepalBillService {

    public static final Logger LOGGER = LoggerFactory.getLogger(RepalBillServiceImpl.class);

    @Autowired
    private RepalBillMapper repalBillMapper;

    @Autowired
    private RepalResponsiblePersonService repalResponsiblePersonService;

    @Autowired
    private RepalGroupMemberService repalGroupMemberService;

    @Autowired
    private RepalCustomerService repalCustomerService;

    @Autowired
    private DataMiddlewareService dataMiddlewareService;

    @Autowired
    private InCaseService inCaseService;

    @Autowired
    private RepalBillDiagnosisService repalBillDiagnosisService;

    @Autowired
    private RepalChangeRecordService repalChangeRecordService;


    @Override
    public VOPage<RepalBillPageVO> getRepalBillPage(GetRepalBillPageDTO dto) {
        if (ObjectUtil.hasEmpty(dto.getRepalGroupId(), dto.getRepalBookId())) {
            throw new CommonException(CodeMsg.VALIDATE_PARAMETER);
        }
        VOPage<RepalBillPageVO> voPage = new VOPage<>();
        voPage.setPageSize(dto.getPageSize());
        voPage.setPageNum(dto.getPageNum());
        // 管理员查看所有   todo
        LOGGER.info("角色:[]");
        Page<RepalBill> repalBillPage = new Page<>(dto.getPageNum(),dto.getPageSize());
        repalBillPage = repalBillMapper.getRepalBillPage(repalBillPage, dto);
        voPage.setTotalPage(Math.toIntExact(repalBillPage.getPages()));
        voPage.setTotalNum(repalBillPage.getTotal());
        if (CollUtil.isEmpty(repalBillPage.getRecords())) {
            voPage.setDataList(new ArrayList<>());
            return voPage;
        }
        List<Long> billIdList = repalBillPage.getRecords().stream().map(RepalBill::getId).collect(Collectors.toList());
        Map<Long, List<String>> nameMap = repalResponsiblePersonService.getResponsiblePersonNameList(billIdList);
        voPage.setDataList(repalBillPage.getRecords().stream().map(t -> {
            RepalBillPageVO vo = new RepalBillPageVO();
            BeanUtils.copyProperties(t, vo);
            vo.setRepalBillId(t.getId());
            vo.setResponsiblePersonNames(String.join("、", nameMap.get(t.getId())));
            return vo;
        }).collect(Collectors.toList()));
        return voPage;
    }

    @Override
    public RepalBillDetailVO getRepalBillDetail(Long repalBillId) {
        RepalBill repalBill = this.getById(repalBillId);
        if (Objects.isNull(repalBill)) {
            return null;
        }
        RepalBillDetailVO vo = new RepalBillDetailVO();
        BeanUtils.copyProperties(repalBill, vo);
        vo.setRepalBillId(repalBill.getId());
        RepalCustomer repalCustomer = repalCustomerService.getById(repalBill.getCustomerId());
        vo.setCompanyId(repalCustomer.getCompanyId());
        vo.setCustProperty(repalCustomer.getCustProperty());
        List<String> nameList = repalResponsiblePersonService.getResponsiblePersonName(vo.getRepalBillId());
        vo.setResponsiblePersonNames(CollUtil.isEmpty(nameList) ? "" : String.join("、", nameList));
        vo.setTotalMoneyStr(repalBill.getTotalMoney().stripTrailingZeros().toPlainString());
        vo.setReceiptStr(repalBill.getReceipt().stripTrailingZeros().toPlainString());
        vo.setReceivableStr(repalBill.getReceivable().stripTrailingZeros().toPlainString());
        // 评分趋势
        vo.setDiagnosisScoreTrend(DiagnosisUtil.getDiagnosisScoreTrend(repalBill.getPreDiagnosisScore(), repalBill.getDiagnosisScore(), repalBill.getPreDiagnosisGrade(), repalBill.getDiagnosisGrade()));
        if (Objects.nonNull(repalBill.getDiagnosisScore())) {
            vo.setDiagnosisGradeDesc(DiagnosisUtil.getGradeConclusion(repalBill.getDiagnosisScore()));
            vo.setDiagnosisScore(repalBill.getDiagnosisScore().stripTrailingZeros().toPlainString());
        }

        // companyId
        if (StrUtil.isNotBlank(vo.getCompanyId())) {
            // 财产数量
            ApiDataResult<List<FinanceDynamicNumVO>> financeResult = dataMiddlewareService.getFinanceDynamicNumV8List(new GetFinanceDynamicNumDTO().setCompanyIdList(Collections.singletonList(vo.getCompanyId())));
            if (Objects.nonNull(financeResult) && CollUtil.isNotEmpty(financeResult.getData())) {
                vo.setAssetsNum(financeResult.getData().get(0).getFinanceDynamicNum());
            }
            // 风险数量
            ApiDataResult<List<CompanyRiskNumVO>> riskResult = dataMiddlewareService.getCompanyRiskNumList(new GetCompanyRiskNumDTO(Collections.singletonList(vo.getCompanyId())));
            if (Objects.nonNull(riskResult) && CollUtil.isNotEmpty(riskResult.getData())) {
                vo.setAssetsNum(riskResult.getData().get(0).getRiskNum());
            }
        }
        if (Objects.nonNull(repalBill.getDiagnosisId())) {
            RepalBillDiagnosis repalBillDiagnosis = repalBillDiagnosisService.getById(repalBill.getDiagnosisId());
            if (Objects.nonNull(repalBillDiagnosis)) {
                ApiDataResult<DiagnosisRecordVO> apiDataResult = inCaseService.getRecordVO(repalBillDiagnosis.getBizId());
                if (Objects.nonNull(apiDataResult)) {
                    vo.setAnalysisList(DiagnosisUtil.convertItemToList(apiDataResult.getData().getResultItem()));
                }
            }
        }
        // 合同
        if (StrUtil.isNotBlank(repalBill.getContractUrl())) {
            List<String> urlList = Arrays.stream(repalBill.getContractUrl().split(";")).collect(Collectors.toList());
            vo.setContractList(urlList.stream().map(s -> {
                RepalBillDetailVO.BillContract billContract = new RepalBillDetailVO.BillContract();
                billContract.setContractUrl(s);
                billContract.setContractName(s.substring(s.lastIndexOf("/") + 1));
                return billContract;
            }).collect(Collectors.toList()));
        }
        if (RepalBillStatusEnum.OVERDUE.getCode().equals(repalBill.getRepalBillStatus())) {
            if (Objects.nonNull(repalBill.getEndDate())) {
                if (LocalDate.now().isAfter(repalBill.getEndDate()) || LocalDate.now().compareTo(repalBill.getEndDate()) == 0) {
                    // 避免天数出现负数
                    vo.setOverdueDay(Math.toIntExact(LocalDate.now().toEpochDay() - repalBill.getEndDate().toEpochDay()) + 1);
                }
            }
        }
        List<RepalChangeRecord> repalChangeRecordList = repalChangeRecordService.list(Wrappers.lambdaQuery(RepalChangeRecord.class)
                .eq(RepalChangeRecord::getRepalBillId, vo.getRepalBillId())
                .eq(RepalChangeRecord::getDeleted, DeletedStatusEnum.NORMAL.getCode()));
        if (CollUtil.isNotEmpty(repalChangeRecordList)) {
            vo.setChangeList(repalChangeRecordList.stream().map(t -> {
                RepalBillDetailVO.BillMoneyChange billMoneyChange = new RepalBillDetailVO.BillMoneyChange();
                billMoneyChange.setChangeTime(t.getTimeCreated());
                billMoneyChange.setChangeDesc(t.getOperUserName() + " 增加回款 " + t.getTransMoney().stripTrailingZeros().toPlainString());
                return billMoneyChange;
            }).collect(Collectors.toList()));
        }
        return vo;
    }

}
